<?php

namespace loyalsoft;

include_once './EventProc/NormalEventProc.php';

/**
 * 任务模块
 * @version
 *          1.0.0 Created at 2017-11-3. by --gwang
 * @author gwang (mail@wanggangzero.cn)
 * @copyright ? 2017-11-3, SJZ LoyalSoft Corporation & gwang. All rights reserved.
 */
class TaskProc {

    static function procMain($req) {
        switch ($req->cmd) {
            case CmdCode::cmd_task_getInfo:                                     # 6201 刷新任务状态
                return TaskProc::GetTaskInfo($req);
            case CmdCode::cmd_task_getReward:                                   # 6202 领取任务奖励
                return TaskProc::GetTaskReward($req);
            case CmdCode::cmd_Task_setAttentionTask:                            # 6203 设置关注任务
                return TaskProc::setAttentionTask($req);
            case CmdCode::cmd_Task_getActiveReward:                             # 6204 领取日常任务活跃度阶段奖励
                return TaskProc::getActiveReward($req);
//---------------- 任务卡相关通讯 -----------------------------------------------
            case CmdCode::cmd_taskCard_PlotFinish:                              # 6210 任务卡剧情对话
                return self::PlotFinish($req);
            case CmdCode::cmd_taskCard_PlotExchangeTaskCard:                    # 6211 任务卡兑换奖励
                return self::PlotExchangeTaskCard($req);
            case CmdCode::cmd_taskCard_PlotPresentTaskCard:                     # 6212 剧情赠送任务卡
                return self::PlotPresetTaskCard($req);
            case CmdCode::cmd_taskCard_FinishAndReward:                         # 6213 任务卡完成,并即时结算
                return self::ImmetRewardTaskCard($req);
            case CmdCode::cmd_taskCard_onKillMonster:                           # 6214 杀死怪物事件
                return self::OnKillMonsterEvent($req);
            default:
                return Err(ErrCode::cmd_err);
        }
    }

    // <editor-fold defaultstate="collapsed" desc="任务卡接口">
    // 
    /**
     * [6210]剧情对话-触发任务卡结束条件检测(中间某个step结束,进入下一个step)
     * @param req $req
     */
    static function PlotFinish($req) {
        // 触发一下任务检查
        list($gate_or_npc_id, $stage) = $req->paras;
        $bUpdate = self::OnPlotOver($gate_or_npc_id, $stage);
        return Resp::ok(array('store' => $req->userInfo->game->store,
                    'taskCardUpdate' => $bUpdate));
    }

    /**
     * [6211] 任务卡-兑换奖励
     * @param type $req
     */
    static function PlotExchangeTaskCard($req) {
        Err(ErrCode::err_method_notimplement);
    }

    /**
     * [6212] 任务卡- 剧情对话奖励任务卡
     * @param req $req
     */
    static function PlotPresetTaskCard($req) {
        // 发放任务卡
        list($rwdStr) = $req->paras;
        $err = StoreProc::AddMultiItemInStore($req, $rwdStr);
        my_Assert(ErrCode::ok == $err, $err);
        UserProc::updateUserInfo();
        return Resp::ok(array('store' => $req->userInfo->game->store));
    }

    /**
     * [6213]任务卡 - 即时奖励型任务卡
     * @param req $req
     */
    static function ImmetRewardTaskCard($req) {
        // 销毁任务卡, 发放奖励
        list($taskCardUID) = $req->paras;
        my_Assert(StlUtil::dictHasProperty($req->userInfo->game->store->taskcards, $taskCardUID), ErrCode::taskCard_no); # 防御
        $taskCard = new TaskCardVo($req->userInfo->game->store->taskcards->$taskCardUID); # 任务卡对象
        if (strlen($taskCard->mo()->reward) > 0) {                              # 防御奖励串为空
            StoreProc::AddMultiItemInStore($req, $taskCard->mo()->reward);      # 发放奖励
        }
        if (strlen($taskCard->mo()->newTaskCard) > 0) {                         # 防御新卡串为空
            StoreProc::AddMultiItemInStore($req, $taskCard->mo()->newTaskCard); # 发放新任务卡
        }
        StlUtil::dictRemove($req->userInfo->game->store->taskcards, $taskCardUID); # 移除任务卡
        UserProc::updateUserInfo();                                             # 回存玩家数据
        return Resp::ok(array('store' => $req->userInfo->game->store, 'taskCardUpdate' => true));         # 返回值更新背包
    }

    /**
     * [6214] 任务卡 - 杀死怪物事件
     * @param req $req
     */
    static function OnKillMonsterEvent($req) {
        // 触发一下任务检查
        list($monsterID, $num) = $req->paras;
        $bUpdate = self::OnKillMonster($monsterID, $num);
        return Resp::ok(array('store' => $req->userInfo->game->store,
                    'taskCardUpdate' => $bUpdate));
    }

    // 
// </editor-fold>
    // <editor-fold defaultstate="collapsed" desc="  任务接口 ">

    /**
     * [6204] 领取日常任务活跃度阶段奖励
     * @param req $req
     */
    static function getActiveReward($req) {
        list($num) = $req->paras;                                               # 提取参数: 领奖的阶段
        $userTask = $req->userInfo->game->task;
        $rewardCfg = GameConfig::task_active_reward_getItem($num);              # 查找奖励配置
        my_Assert(null != $rewardCfg, ErrCode::err_const_no);                   # 防御: 常量丢失
        my_Assert(!in_array($num, $userTask->dailyActiveGetedRewardArr), ErrCode::task_reward_geted); # 判断尚未领取
        my_Assert($userTask->dailyActivePoint >= $num, ErrCode::task_activePoint_notenough);          # 判断活跃点数足够
        $ok = StoreProc::AddMultiItemInStore($req, $rewardCfg->reward);         # 按照配置发放奖励
        my_Assert($ok == ErrCode::ok, $ok);                                     # 发放成功
        $userTask->dailyActiveGetedRewardArr[] = $num;                          # 添加领取记录
        UserProc::updateUserInfo();                                             # 回存数据
        return Resp::ok(array(//                                                # 返回
                    "gold" => $req->userInfo->game->baseInfo->gold,
                    "cash" => $req->userInfo->game->baseInfo->cash,
                    "tili" => $req->userInfo->game->baseInfo->tili,
                    "store" => $req->userInfo->game->store,
                    "task" => $req->userInfo->game->task
        ));
    }

    /**
     * [6203] 设置关注任务
     * @param req $req
     */
    static function setAttentionTask($req) {
        list($taskId) = $req->paras;
        $userTask = $req->userInfo->game->task;

        my_Assert(StlUtil::dictHasProperty($userTask->taskListPlot, $taskId), ErrCode::task_no); # 任务存在
        $taskItem = new TaskInfo($userTask->taskListPlot->$taskId);
        my_Assert($taskItem->rewardGeted < 1, ErrCode::task_reward_geted);      # 任务奖励尚未领取
        $userTask->curTaskId = $taskId;
//        var_dump($userTask);
        UserProc::updateUserInfo();
        return Resp::ok(array("task" => $req->userInfo->game->task));
    }

    /**
     * [6202]领取任务奖励 问:每日任务奖励和剧情任务奖励领取接口是否为一个?
     * @param req $req
     */
    static function GetTaskReward($req) {
        list($taskId) = $req->paras;                                            # 提取参数: 任务id
        $userTask = new UserTaskInfo($req->userInfo->game->task);
        $mask = 100000;                                                         # 根据id段来区分任务和每日任务.
        $type = intval($taskId / $mask);                                        # 取出taskId第一位判断任务类型
        switch ($type) {                                                        # 
            case 4:                                                             # 剧情任务
                my_Assert(StlUtil::dictHasProperty($userTask->taskListPlot, $taskId), ErrCode::task_no); #判断是否存在指定任务
                $task = new TaskInfo($userTask->taskListPlot->$taskId);
                my_Assert($task->progress >= 1, ErrCode::task_progress_not_complete);   # 判断任务进度是否已完成>=1f
                my_Assert($task->rewardGeted < 1, ErrCode::task_reward_geted);          # 奖励不可重复领取
                $taskCfg = GameConfig::task_getItem($taskId);                           # 任务配置数据
                my_Assert($taskCfg != null, ErrCode::err_const_no);
                $ok = StoreProc::AddMultiItemInStore($req, $taskCfg->reward);           # 发放任务奖励
                my_Assert($ok == ErrCode::ok, $ok);
                $task->rewardGeted = 1;
                $userTask->taskListPlot->$taskId = $task;
                if ($userTask->IsAllPlotTaskOver()) {                           # 检查当前节点任务是否已经全部清空
                    $userTask->curTaskNode += 1;
                    $userTask->taskListPlot = ObjectInit();
                    $userTask->initialize();
                }
                break;
            case 5:                                                             # 每日任务
                my_Assert(StlUtil::dictHasProperty($userTask->taskListDaily, $taskId), ErrCode::task_no);
                $task = new TaskInfo($userTask->taskListDaily->$taskId);
                my_Assert($task->progress >= 1, ErrCode::task_progress_not_complete);   # 判断任务进度是否已完成>=1f
                my_Assert($task->rewardGeted < 1, ErrCode::task_reward_geted);          # 奖励不可重复领取
                $taskCfg = GameConfig::task_daily_getItem($taskId);                     # 任务配置数据
                my_Assert($taskCfg != null, ErrCode::err_const_no);
                UserGameModel::Add_Exp($req->userInfo->game->baseInfo, $taskCfg->exp);  # 发放任务奖励 
                $userTask->dailyActivePoint += $taskCfg->activePoint;
                $task->rewardGeted = 1;
                $userTask->taskListDaily->$taskId = $task;                      # 回存下
                break;
            default:
                break;
        }

        $req->userInfo->game->task = $userTask;                                 # 回存 
        UserProc::updateUserInfo();
        return Resp::ok(array(//                                                # 返回
                    "gold" => $req->userInfo->game->baseInfo->gold,
                    "cash" => $req->userInfo->game->baseInfo->cash,
                    "tili" => $req->userInfo->game->baseInfo->tili,
                    "store" => $req->userInfo->game->store,
                    "task" => $req->userInfo->game->task
        ));
    }

    /**
     * [6201]更新任务状态
     * @param req $req
     */
    static function GetTaskInfo($req) {
        return Resp::ok(array("task" => $req->userInfo->game->task));
    }

// </editor-fold>

    /**
     * 重置每日任务
     * @param Req $req
     */
    static function ResetDailyTask($req) {
        $userTask = new UserTaskInfo($req->userInfo->game->task);
        $userTask->resetDailyTask();
        $req->userInfo->game->task = $userTask;
    }

// 
    // <editor-fold defaultstate="collapsed" desc="辅助方法-比对判断">

    /**
     * 检查任务条件是否达成
     * @param TaskParams $taskParam Description
     */
    static function CheckTaskConditions($taskParam) {
//        var_dump($taskParam);
        $bUpdate = false;
        foreach (req()->userInfo->game->task->taskListPlot as $tid => $task) {
            $taskCfg = GameConfig::task_getItem($tid);
            my_Assert($taskCfg != null, ErrCode::err_const_no);                 # 获取任务配置数据   
            if ($taskCfg->tasktype != $taskParam->taskType) {                   # 任务类型必须匹配
                continue;
            }
            isEditor() and $task = new TaskInfo(-1);
            if ($task->progress == 1) {                                         # 已完成的不再更新处理
                continue;
            }
            if (is_null($taskCfg->canshu1)) {
                $task->progress = 1;
                $bUpdate = true;
                continue;
            }

            if ($taskCfg->canshu1 == $taskParam->canshu1) {
                $task->tag1 = $taskParam->canshu1;
                if (null == $taskCfg->canshu2 || 0 == $taskCfg->canshu2) {
                    $task->progress = 1;
                    $task->tag2 = $taskParam->canshu2;
                    $bUpdate = true;
                } else {
                    if (is_numeric($taskCfg->canshu2)) {
                        $task->tag2 = max($task->tag2, $taskParam->canshu2);
                        if ($taskCfg->canshu2 <= $taskParam->canshu2) {
                            $bUpdate = true;
                            $task->progress = 1;
                        }
                    } else {
                        $task->tag2 = $taskParam->canshu2;
                        if ($taskCfg->canshu2 == $taskParam->canshu2) {
                            $bUpdate = true;
                            $task->progress = 1;
                        }
                    }
                }
            }
        }
        if ($bUpdate) {
            Resp::addTag("isTaskUpdated", true);                                # 附加tag信息, 任务有更新
            CornerSignEventProc::OnTask_Plot_new(req());
        }
    }

    /**
     * 检查任务条件是否达成
     * @param TaskParams $taskParam Description
     */
    static function CheckDailyTaskConditions($taskParam) {
        $bUpdate = false;
        $daily = req()->userInfo->game->task->taskListDaily;
        foreach ($daily as $tid => &$task) {
            $taskCfg = GameConfig::task_daily_getItem($tid);
            my_Assert($taskCfg != null, ErrCode::err_const_no);                 # 获取任务配置数据    
            if ($taskCfg->tasktype != $taskParam->taskType) {                   # 任务类型必须匹配
                continue;
            }
            $task = new TaskInfo($task);
            if ($task->progress >= 1) {                                         # 已完成的不再更新处理
                continue;
            }
            $task->counter++;                                                   # 计数器
            if (is_null($taskCfg->canshu1) || $taskCfg->canshu1 == 0) {
                $task->progress = 1;
                $bUpdate = true;
                continue;
            }
            if ($taskCfg->canshu1 == $task->counter) {
                $task->progress = 1;
                $bUpdate = true;
                continue;
            }
        }

        if ($bUpdate) {
            Resp::addTag("isTaskUpdated", true);                                # 附加tag信息, 任务有更新
            CornerSignEventProc::OnTask_Daily_new(req());
        }
    }

// </editor-fold>
// 
    // <editor-fold defaultstate="collapsed" desc="处理各种任务检测事务">
    // 

    /**
     * 例如当指挥官等级提升的时候需要检查是否有任务达成
     */
    static function OnUserLevelUp($newLvl) {
        $taskParam = new TaskParams(EnumTaskType::CommanderLevelUpTo, $newLvl);
        self::CheckTaskConditions($taskParam);
    }

    /**
     * 英雄升级
     * @param type $heroMoId
     * @param type $newLvl
     */
    static function OnHeroLevelUp($heroMoId, $newLvl) {
        $taskParam = new TaskParams(EnumTaskType::HeroLevelUpTo, $heroMoId, $newLvl);
        self::CheckTaskConditions($taskParam);
    }

    /**
     * 英雄升阶
     * @param type $heroMoId
     * @param type $newGrade
     */
    static function OnHeroGradeUp($heroMoId, $newGrade) {
        $taskParam = new TaskParams(EnumTaskType::HeroGradeUpTo, $heroMoId, $newGrade);
        self::CheckTaskConditions($taskParam);
        self::OnHeroImprove();
    }

    /**
     * 英雄碎片数量达到指定值
     * @param type $heroMoId
     * @param type $num
     */
    static function OnHeroSegmengNum($heroMoId, $num) {
        $taskParam = new TaskParams(EnumTaskType::HeroSegmentNumberTo, $heroMoId, $num);
        self::CheckTaskConditions($taskParam);
    }

    /**
     * 英雄解锁技能
     * @param type $heroMoId
     * @param type $skillId
     */
    static function OnHeroUnlocSkill($heroMoId, $skillId) {
        $taskParam = new TaskParams(EnumTaskType::HeroSkillUnlock, $heroMoId, $skillId);
        self::CheckTaskConditions($taskParam);
    }

    /**
     * 通关剧情关卡
     * @param type $gateId
     * @param type $difficulty
     */
    static function OnPassGateN($gateId, $difficulty) {
        $taskParam = new TaskParams(EnumTaskType::PassGateN, $gateId, $difficulty);
        self::CheckTaskConditions($taskParam);
        self::OnPassGate();
    }

    /**
     * 通关副本关卡
     * @param type $gateId
     * @param type $difficulty
     */
    static function OnPassCarboN($gateId, $difficulty) {
        $taskParam = new TaskParams(EnumTaskType::passCarbonN, $gateId, $difficulty);
        self::CheckTaskConditions($taskParam);
    }

    /**
     * 竞技场胜利场次
     * @param type $num
     */
    static function OnPvPWinN($num) {
        $taskParam = new TaskParams(EnumTaskType::PvPWinNumberTo, $num);
        self::CheckTaskConditions($taskParam);
    }

    /**
     * 竞技场积分
     * @param type $score
     */
    static function OnPvPScoreN($score) {
        $taskParam = new TaskParams(EnumTaskType::PvPScoreTo, $score);
        self::CheckTaskConditions($taskParam);
    }

    /**
     * 玩家总战力值
     * @param type $num
     */
    static function OnUserFightPowerN($num) {
        $taskParam = new TaskParams(EnumTaskType::UserPowerTo, $num);
        self::CheckTaskConditions($taskParam);
    }

    /**
     * 玩家收集积分
     * @param type $num
     */
    static function OnUserCollectScoreN($num) {
        $taskParam = new TaskParams(EnumTaskType::UserCollectScoreTo, $num);
        self::CheckTaskConditions($taskParam);
    }

    // 
// </editor-fold>
    //
    // <editor-fold defaultstate="collapsed" desc="每日任务条件检查">
    // 

    /**
     * 每日登陆
     */
    static function OnUserLogin() {
        $taskParam = new TaskParams(EnumTaskType::DailyLogin);
        self::CheckDailyTaskConditions($taskParam);
    }

    /**
     * 每日:英雄强化(升级或升阶)n次
     */
    static function OnHeroImprove() {
        $taskParam = new TaskParams(EnumTaskType::DailyHeroImprove);
        self::CheckDailyTaskConditions($taskParam);
    }

    /**
     * 每日:剧情关卡,胜利n次
     */
    static function OnPassGate() {
        $taskParam = new TaskParams(EnumTaskType::DailyGatesWin);
        self::CheckDailyTaskConditions($taskParam);
    }

    /**
     * 竞技场每日挑战,胜负均可
     */
    static function OnPvp() {
        $taskParam = new TaskParams(EnumTaskType::DailyPVP);
        self::CheckDailyTaskConditions($taskParam);
    }

    /**
     * 每日挑战,胜负均可
     */
    static function OnRankChalenge() {
        $taskParam = new TaskParams(EnumTaskType::DailyRankChalenge);
        self::CheckDailyTaskConditions($taskParam);
    }

    /**
     * 每日购买商品
     */
    static function OnShopping() {
        $taskParam = new TaskParams(EnumTaskType::DailyShopping);
        self::CheckDailyTaskConditions($taskParam);
    }

    // 
// </editor-fold>
    // 
    // <editor-fold defaultstate="collapsed" desc="处理任务卡条件检查">
    // 

    /**
     * 检查任务卡条件是否达成
     * @param TaskParams $taskParam Description
     */
    static function CheckTaskCardConditions($taskParam) {
        $bUpdate = false;
        $tasks = req()->userInfo->game->store->taskcards;

        foreach ($tasks as $tid => &$task) {
            $task = new TaskCardVo($task);
            $taskCfg = $task->mo();                                             # 获取任务卡配置数据
            // 根据steptype类型区分下, 然后取任务步骤数据, 比对任务步骤的进度
            if ($taskCfg->stepType == 1) {                                      # 1串,2并
                $taskSteps = $task->curSteps;                                   # 任务步骤
                $n = $task->curStepIndex;
                $taskStep = new TaskStepVo($taskSteps[$n]);
                $bUpdate = $taskStep->check($taskParam);
                if ($bUpdate) {
                    NormalEventProc::OnTaskCardStep_Complete($task->typeId, $taskStep->typeId);
                    $taskSteps[$n] = $taskStep;
                    $task->curSteps = $taskSteps;
                    $n++;
                    if (count($task->curSteps) <= $n) {
                        $task->state = 1;                                       # 任务已完成
                        NormalEventProc::OnTaskCard_Finish($task->typeId, null);
                    } else {
                        $task->curStepIndex = $n;
                    }
                }
            } else {                                                            # 并
                foreach ($task->curSteps as $stp) {
                    $tsp = new TaskStepVo($stp);
                    $bUpdate = $tsp->check($taskParam);
                }
            }
        }
        if ($bUpdate) {
            UserProc::updateUserInfo();
        }

        return $bUpdate;
    }

    static function OnKillMonster($monsterID, $num) {
        $taskParam = new TaskParams(EnumTaskType::KillMonster, $monsterID, $num);
        return self::CheckTaskCardConditions($taskParam);
    }

    /**
     * 获得道具
     */
    static function OnGainItem($itemId, $num) {
        $taskParam = new TaskParams(EnumTaskType::GainItem, $itemId, $num);
        self::CheckTaskCardConditions($taskParam);
        NormalEventProc::OnBag_new_Item($itemId, $num);
    }

    static function OnPlotOver($gate_or_npc_id, $stage) {
        $taskParam = new TaskParams(EnumTaskType::PlotOver, $gate_or_npc_id, "$stage");
        return self::CheckTaskCardConditions($taskParam);
    }

    // 
    // </editor-fold>
//
}
